# -*- Mode: perl; indent-tabs-mode: nil -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (C) 2013 Jolla Ltd.
# Contact: Pami Ketolainen <pami.ketolainen@jollamobile.com>

=head1 NAME

Bugzilla::Extension::AgileTools::WebService::Backlog

=head1 DESCRIPTION

Web service methods for managing backlogs.
Namespace: Agile.Backlog

=cut

use strict;
use warnings;
use 5.10.1;

package Bugzilla::Extension::AgileTools::WebService::Backlog;

use base qw(Bugzilla::WebService);

use Bugzilla::Constants;
use Bugzilla::Error;

use Bugzilla::Extension::AgileTools::Backlog;
use Bugzilla::Extension::AgileTools::Team;

use Bugzilla::Extension::AgileTools::Util;
use Bugzilla::Extension::AgileTools::WebService::Util;

# Webservice field type mapping
use constant FIELD_TYPES => {name => "string", team_id => "int",};

use constant PUBLIC_METHODS => qw(
  update
  create
);

=head1 METHODS

=over

=item C<update>

    Description: Updates Backlog
    Params:      id - Backlog id
                 team_id - New Team ID
    Returns:     { backlog => backlog object
                   changes => Changes hash from Bugzilla::Object::update }

=cut

sub update {
  my ($self, $params) = @_;
  Bugzilla->login(LOGIN_REQUIRED);
  user_in_agiletools_group(1);
  my $backlog_id = delete $params->{id};
  ThrowCodeError('param_required',
    {function => 'Agile.Backlog.update', param => 'id'})
    unless defined $backlog_id;

  my $backlog
    = Bugzilla::Extension::AgileTools::Backlog->check({id => $backlog_id});

  ## RED HAT EXTENSION START 1323078
  # ensures user has access to team
  my $team;
  $team = get_team($backlog->team_id, 1) if($backlog->team_id);
  ## RED HAT EXTENSION END 1323078

  if (exists $params->{team_id}) {

    # Check that user is allowed to edit both teams
    my $new_team = $params->{team_id} || 0;
    my $old_team = $backlog->team_id  || 0;
    if ($new_team != $old_team) {
      get_team($new_team, 1) if $new_team;
      get_team($old_team, 1) if $old_team;
    }
  }

  $backlog->set_all($params);
  my $changes = $backlog->update();
  return {
    backlog => $backlog,
    changes => changes_to_hash($self, $changes, FIELD_TYPES)
  };
}

=item C<create>

    Description: Create new Backlog
    Params:      name - Backlog name
                 team_id - (optional) Team ID for the backlog
    Returns:     The new Backlog object

=cut

sub create {
  my ($self, $params) = @_;
  Bugzilla->login(LOGIN_REQUIRED);
  user_in_agiletools_group(1);

  my $team = undef;
  $team = get_team($params->{team_id}, 1) if (defined $params->{team_id});

  my $backlog = Bugzilla::Extension::AgileTools::Backlog->create($params);
  return $backlog;
}

1;

__END__

=back

=head1 SEE ALSO

L<Bugzilla::WebService>

