# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

package Bugzilla::Extension::AuthJWT::Util;

use 5.10.1;
use strict;
use warnings;

use Bugzilla::Extension::AuthJWT::Source;
use JSON;
use Bugzilla::User;
use List::MoreUtils qw(any);

use parent qw(Exporter);
our @EXPORT = qw(
  filter_mail
  sources_kids
  user_can_use
);

# This file can be loaded by your extension via
# "use Bugzilla::Extension::AuthJWT::Util". You can put functions
# used by your extension in here. (Make sure you also list them in
# @EXPORT.)

sub sources_kids {
  my %sources;

  foreach my $src (@{Bugzilla::Extension::AuthJWT::Source->match()}) {
    $sources{$src->kid} = $src->cert;
  }

  my $sources = encode_json(\%sources);
  return ($sources);
}

sub user_can_use {
  my ($kid, $user) = @_;

  my $can    = 1;
  my $source = Bugzilla::Extension::AuthJWT::Source->match({kid => $kid});
  $can = 0
    if (!@$source || any { $user->in_group_id($_) } @{$source->[0]->groups()});

  return ($can);
}

## If the JWT source has a regex set, then filter the email address based on it.
sub filter_mail {
  my ($kid, $mail) = @_;

  my $can    = 1;
  my $source = Bugzilla::Extension::AuthJWT::Source->match({kid => $kid});

  if (@$source && $source->[0]->regex() ne '') {
    my $regex = $source->[0]->regex();
    my $repl  = $source->[0]->repl();
    $mail =~ s/$regex/$repl/;
  }

  return ($mail);
}

1;
