# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

package Bugzilla::Extension::ExternalBugs::Type::Gerrit;

use strict;
use warnings;
use 5.10.1;

use base qw(Bugzilla::Extension::ExternalBugs::Type);

use Bugzilla::Constants;
use Bugzilla::Error;
use Bugzilla::Util qw(correct_urlbase);

use Data::Dumper;
use URI;
use LWP::UserAgent;
use MIME::Base64;
use HTTP::Request;

use constant TYPE => 'Gerrit';

sub get_data {
  my ($self, $ext_bug) = @_;

  my $url  = $ext_bug->type->url;
  my $last = chop($url);
  $url .= $last if ($last ne '/');
  $url .= "/changes/" . $ext_bug->ext_bz_bug_id;

  my $data = $self->do_rest_call($url, 'GET', qr|\)\]\}'|);
  my $bug_data;

  # Get the information we require from the result
  if ($data->{status}) {
    $bug_data->{status} = $data->{status};
  }
  if ($data->{subject}) {
    $bug_data->{description} = $data->{subject};
  }

  return $bug_data;
}

sub can_sync {
  my ($self, $ext_bug) = @_;
  return 1;
}

1;

__END__

=head1 NAME

Bugzilla::Extension::ExternalBugs::Type::Gerrit

=head1 DESCRIPTION

Type represents a Gerrit review.  It is an implementation of
L<Bugzilla::Extension::ExternalBugs::Type>, and thus provides all methods that
L<Bugzilla::Extension::ExternalBugs::Type> provides.

This type can sync data from remote Gerrit sites to the local site.

=cut
