# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

package Bugzilla::Extension::ExternalBugs::Type::KBase;

use strict;
use warnings;
use 5.10.1;

use base qw(Bugzilla::Extension::ExternalBugs::Type);

use Bugzilla::Constants;
use Bugzilla::Util qw(correct_urlbase);
use Bugzilla::Error;

use JSON;
use URI;
use HTTP::Request;
use LWP::UserAgent;

use constant TYPE => 'KBase';

my $logger = Bugzilla->logger;

sub get_data {
  my ($self, $ext_bug) = @_;
  my $bug_data;

  my $url = $ext_bug->type->url;
  $url =~ s{access.redhat.com}{strata.ext.paas.redhat.com/rs};
  $url .= $ext_bug->ext_bz_bug_id . '&isPublished=true';
  my $uri = URI->new($url);
  my $ua  = LWP::UserAgent->new;
  $ua->agent("RedHatBugzilla/" . BUGZILLA_VERSION);    # :)

  my $result = undef;

  $logger->debug("Bugzilla RPC Calling: $url");

  # Create a request
  my $req = HTTP::Request->new('GET', $uri, ['Accept' => 'application/json',],);

  # Pass request to the user agent and get a response back
  my $res = $ua->request($req);

  unless ($res->is_success) {
    die $res->status_line;
  }

  # Some calls return no content
  $result = $res->content ? decode_json($res->content) : '';

  if ($result->{'title'}) {
    $bug_data                  = {};
    $bug_data->{'priority'}    = $result->{'categories'}{'category'}[0];
    $bug_data->{'description'} = $result->{'title'};
  }

  return $bug_data;
}

sub can_sync {
  my ($self, $ext_bug) = @_;
  return 1;
}

1;

__END__

=head1 NAME

Bugzilla::Extension::ExternalBugs::Type::KBase

=head1 DESCRIPTION

Type represents a Red Hat KBase article. It is an implementation of
L<Bugzilla::Extension::ExternalBugs::Type>, and thus provides all methods that
L<Bugzilla::Extension::ExternalBugs::Type> provides.

This type can sync data from remote KBase sites to the local site.

=cut
