#!/usr/bin/perl
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

use strict;
use warnings;
use 5.10.1;

# Bug 1334571 - Job queue monitoring script
# Monitor the number of rows inserted vs the number of rows
# deleted.  If we notice that rows are not being continually
# deleted, then we can say that the job queue is not correctly
# processing jobs, and should be restarted.

use lib qw(/var/www/html/bugzilla);

use Bugzilla;

my $dbh = Bugzilla->dbh;

my $sql = << 'EOF'
    SELECT
        pg_stat_get_tuples_inserted(c.oid) AS inserted,
        pg_stat_get_tuples_deleted(c.oid) AS deleted
    FROM
        pg_class c
    WHERE
        relname = 'ts_job'
EOF
  ;

my ($inserted, $deleted) = @{$dbh->selectall_arrayref($sql)->[0]};

say "Inserted: $inserted";
say "Deleted : $deleted";
