#!/usr/bin/perl
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

use strict;
use warnings;
use 5.10.1;

use lib qw(. /var/www/html/bugzilla/);

=head1 NAME

import-metadata.pl - Import an IdP's metadadta from an XML file.

=head1 SYNOPSIS

 import-metadata.pl "<IDP Name>" $path-to-metadata.xml

=head1 DESCRIPTION

 This script takes the Name of the IdP to update and a path to the metadata XML file.
 It will import th XML in to the IdPs database entry to be used for connections to that IdP.

=cut

use DateTime;
use File::Slurp qw( read_file );
use open ':encoding(utf8)';
use Carp;

use Bugzilla;

my $name = $ARGV[0] || carp("you need to supply a name and a path");
my $xml  = read_file($ARGV[1]);

my $dbh = Bugzilla->dbh;

$dbh->do(
  q{UPDATE saml2auth_idp SET metadata = ?, expires =  date_trunc('day', NOW() + interval '10 year') WHERE name = ?},
  undef, $xml, $name
);

exit;

