# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

package Bugzilla::Extension::SAML2Auth::Verify;
use 5.10.1;
use strict;
use warnings;
use base qw(Bugzilla::Auth::Verify);

use Bugzilla::Constants qw( AUTH_NO_SUCH_USER );

use constant {user_can_create_account => 1, extern_id_used => 1};

sub check_credentials {
  my ($self, $login_data) = @_;

  return ({failure => AUTH_NO_SUCH_USER}) unless ($login_data->{saml2_auth});

  return ($login_data);
}

sub create_or_update_user {
  my ($self, $params) = @_;

  my $couu_user = $self->SUPER::create_or_update_user($params);

  my $user = $couu_user->{user};

  if ($params->{force_enable}) {
    $user->set_disabledtext('');
    $user->set_email_enabled(1);
    $user->set_password('*');
    $user->update();
  }

  return {user => $user};
}

1;

__END__

=head1 Description

Bugzilla::Extension::SAML2Auth::Verify - A module to verify SAML2 Login has succeeded.

=head1 Methods

=over 4

=item check_credentials

Ensure the SAML details have matched a Bugzilla account.

=item create_or_update_user

Override super method to allow reenabling accounts.

=back


